/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.ftp;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.objectweb.petals.component.framework.PEtALSCFException;
import org.objectweb.petals.component.framework.util.SourceHelper;

/**
 * Some IO utilities for the file transfer BC.
 *
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.2
 * @author eBM WebSourcing
 *
 */
public final class FTPUtils {

	/**
	 * Sends the content to a ftp server. All parameters must be non null
	 *
	 * @param fileName
	 *            the file name
	 * @param content
	 *            the content
	 * @param server
	 *            the server
	 * @param port
	 *            the port
	 * @param login
	 *            the login
	 * @param password
	 *            the password
	 * @param destination
	 *            the destination
	 * @throws FTPBCException
	 *             impossible to send
	 * @return true if ok
	 */
	public static boolean put(final String fileName, final DataHandler content,
			final String server, final int port, final String login,
			final String password, final String folder) throws IOException {
		boolean result = false;
		FTPClient client = null;
		try {
			client = FTPClientPool.createNewFTPClient(server, port, login, password);
			String destination = null;

			if (folder.length() > 0) {
				destination = folder + "/" + fileName;
			} else {
				destination = fileName;
			}
			OutputStream ops = client.storeFileStream(destination);
			content.writeTo(ops);
			ops.flush();
			ops.close();
		} catch (IOException ioe) {
			throw ioe;
		} finally {
			client.disconnect();
		}


		return result;

	}
	public static boolean putSource(final String fileName, Source content,
			final String server, final int port, final String login,
			final String password, final String folder) throws IOException{
		FTPClient client = null;
		boolean result = false;
		try {
			client = FTPClientPool.createNewFTPClient(server, port, login, password);
			String destination = null;

			if (folder.length() > 0) {
				destination = folder + "/" + fileName;
			} else {
				destination = fileName;
			}
			OutputStream ops = client.storeFileStream(destination);
			StreamResult streamResult = new StreamResult(ops);
			Transformer transform = createNewTransformer();
			try {
				transform.transform(content, streamResult);
			} catch (TransformerException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}




		} catch (IOException ioe) {
			throw ioe;
		} finally {
			client.disconnect();
		}

		return result;

	}

	/**
	 * Gets a file from a ftp server.
	 *
	 * @param server
	 *            the server
	 * @param port
	 *            the port
	 * @param login
	 *            the login
	 * @param password
	 *            the password
	 * @param remoteFile
	 *            the remote file
	 *
	 * @return String
	 * @throws FTPBCException
	 *             impossible to get: The file does not exist
	 */
	public static DataHandler get(final String server, final int port,
			final String login, final String password, final String remoteFile)
			throws IOException {
		DataHandler result;
		FTPClient client = null;
		try {
			client = FTPClientPool.createNewFTPClient(server, port, login, password);
			result = get(remoteFile,client);
		} catch (IOException e) {
			throw e;
		} finally {
			try {
				client.disconnect();
			} catch (IOException e) {
				// Do nothing
			}

		}
		return result;
	}

	/**
	 * this method will only get files, it does not include subpath
	 * @param server
	 * @param port
	 * @param login
	 * @param password
	 * @param destination
	 * @return
	 * @throws IOException
	 */
	public static Map<String, DataHandler> mGet (final String server, final int port,
			final String login, final String password, String destination)
	throws IOException {

		FTPClient client = null;
		client = FTPClientPool.createNewFTPClient(server, port, login, password);
		Map<String, DataHandler> datahandlers = new HashMap<String, DataHandler>();

		try{
			FTPFile[] files = client.listFiles(destination);
			for (FTPFile file : files) {
				if(file.isFile()){
					datahandlers.put(file.getName(),get(file.getName(), client));
				}
			}
		} catch (IOException e) {
			throw e;
		} finally {
			try {
				client.disconnect();
			} catch (IOException e) {
				// Do nothing
			}

		}
		return datahandlers;
	}

	/**
	 * The ftp-client MUST be connected.
	 * This method DOES NOT close the ftp-client
	 * @param remoteFile
	 * @param client a ftpclient, MUST be CONNECTED
	 * @return the retieved file, as a datahandler
	 * @throws IOException the file is not found or FTP access problem
	 */
	public static DataHandler get(String remoteFile, FTPClient client)throws IOException {
			DataHandler result;

			InputStream in = client.retrieveFileStream(remoteFile);


			StreamDataSource datasource = new StreamDataSource(in, remoteFile);
			result = new DataHandler(datasource);

		return result;
	}


	/**
	 * detects if a file exists on a ftp server, or if a directory is not empty.
	 *
	 * @param server
	 *            the server
	 * @param port
	 *            the port
	 * @param login
	 *            the login
	 * @param password
	 *            the password
	 * @param destination
	 *            the destination
	 *
	 * @return true if OK
	 * @throws FTPBCException
	 *             impossible to detetc
	 */
	public static List<String> list(final String server, final int port,
			final String login, final String password, final String destination)
			throws IOException {
		FTPClient client = null;
		List<String> fileList = new ArrayList<String>();

		try {
			client = FTPClientPool.createNewFTPClient(server, port, login, password);
			FTPFile[] files = client.listFiles(destination);
			for (FTPFile file : files) {
				if(file.isFile()){
					fileList.add(file.getName());
				}
			}
		} catch (IOException e) {
			throw e;
		} finally {

			try {
				client.disconnect();
			} catch (IOException e) {
				// Do nothing
			}

		}
		return fileList;
	}

	/**
	 * Create a new transformer.
	 *
	 * @return
	 */
		private static Transformer createNewTransformer() {
			Transformer transformer = null;
			try {
				transformer = TransformerFactory.newInstance().newTransformer();
			} catch (TransformerConfigurationException e) {
				// logger.error("Can't create transformer pool", e);
			} catch (TransformerFactoryConfigurationError e) {
				// logger.error("Can't create transformer pool", e);
			}
			Properties props = new Properties();
			props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
			if (transformer != null) {
				transformer.setOutputProperties(props);
			}
			return transformer;
		}

	public static void main(String[] args) {
		try {
			FTPClient client= FTPClientPool.createNewFTPClient("www.ebmwebsourcing.org", 21, "petalsdemo", "mac125pkg!");

			InputStream in = client.retrieveFileStream("nomFichier.xml");

			StreamSource s = new StreamSource(in);
			System.out.println(SourceHelper.createString(s));
			client.disconnect();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PEtALSCFException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
