package org.objectweb.petals.binding.ftp.listeners;

import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;

import javax.activation.DataHandler;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.binding.ftp.FTPSUProperties;
import org.objectweb.petals.binding.ftp.FTPUtils;
import org.objectweb.petals.component.framework.listener.AbstractJBIListener;
import org.objectweb.petals.component.framework.util.Exchange;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 * @version
 * @since Petals 2.0
 * @author eBMWebsourcing
 *
 */

public class FTPBCJBIListener extends AbstractJBIListener {

	/**
	 * process the exchange depending on the operation:
	 * <li> <b>dir</b> return the filenames list or the remote directory </li>
	 * <li> <b>get</b> return the remote file with the name specified in the
	 * exchange (as attachment or source, depending on the SU property) </li>
	 * <li> <b>mget</b> return all the remote files
	 * <li> <b>put</b> put the message (with the name specified in the su) and
	 * all the attachments </li>
	 */
	public void onJBIMessage(Exchange exchange) {
		try {
			String operationexchange = exchange.getOperationName();
			if (operationexchange==null || "".equals(operationexchange))
				operationexchange = getExtensions()
						.getPetalsExtensions().getProperty(FTPSUProperties.FTP_OPERATION);
			if (operationexchange==null || "".equals(operationexchange))
				operationexchange = "put";
			if ("dir".equals(operationexchange)) {
				// "list" operation used to detect if a remote file exists
				// or if a remote directory contains files
				processDir(exchange);
			} else if ("get".equals(operationexchange)) {
				// "get" operation is used to get a file from the ftp
				// server.
				// The file is returned ss attachment to the response
				// message
				processGet(exchange);
			} else if ("mget".equals(operationexchange)) {
				// "get" operation is used to get a file from the ftp
				// server.
				// The file is returned ss attachment to the response
				// message
				processMGet(exchange);
			} else if ("put".equals(operationexchange)) {
				// "put" default operation is used to send files
				// (send Source and/or attachment, depending on the su
				// property)
				// to the ftp server
				processPut(exchange);
			} else
					exchange.setFault(new Exception("The operation "+operationexchange+" is not recognized"));
			} catch (Exception e) {
				getLogger().log(Level.SEVERE, e.getMessage(), e);
				exchange.setError(new Exception(e.getMessage()));
			}

	}
	/**
	 * FTP PUT operation <br>
	 * Depending on the su property <b>ftp.put-format</b> Possible values :
	 * <li><b>source</> , put the source message as a file</li>
	 * <li><b>attachments</> , put all the attachments</li>
	 * <li><b>source-and-attachments</> , put the source and all the
	 * attachments</li>
	 *
	 * @param exchange
	 *            the message exchange, MUST be non null
	 * @throws Exception
	 *             impossible to realize the PUT operation, or pattern is InOut
	 */
	private void processPut(final Exchange exchange)
			throws Exception {
		if (exchange == null)
			throw new NullPointerException("exchange parameter is null");

		getLogger().info("process PUT operation");
		if (!exchange.isInOutPattern()) {

			String putFormat = getExtensions().getPetalsExtensions()
					.getProperty("ftp.put.exchange-data");

			if ("source".equalsIgnoreCase(putFormat)) {
				sendSource(exchange.getInMessageContentAsSource());
			} else if ("attachments".equalsIgnoreCase(putFormat)) {
				for (Object attachmentName : exchange.getInMessage()
						.getAttachmentNames()) {
					putAttachment(attachmentName.toString(), exchange
							.getInMessage().getAttachment(
									attachmentName.toString()));
				}
			} else if("source-and-attachments".equals(putFormat)) {
				sendSource(exchange.getInMessageContentAsSource());

				for (Object attachmentName : exchange.getInMessage()
						.getAttachmentNames()) {
					putAttachment(attachmentName.toString(), exchange
							.getInMessage().getAttachment(
									attachmentName.toString()));
				}

			}
			else{
				exchange.setFault(new Exception(putFormat+" is not recognize ftp.put.exchange-data value"));
			}
		} else {
			throw new Exception(
					"Exchange pattern for PUT operation can not be In-Out");
		}
	}
/**
 *
 *FTP GET operation <br>
 * Depending on the su property <b>ftp.get-format</b> Possible values :
 * <li><b>source</> , get the file as a source message</li>
 * <li><b>attachments</> , get the file as a attachment</li>
 *
 * @param exchange
 *  		the message exchange, MUST be non null
 * @throws Exception
 * 			impossible to realize the GET operation, or pattern is not InOut or InOptionalOut
 */
	private void processGet(final Exchange exchange)
			throws Exception {
		// get accepts only InOut and InOptionalOut message patterns
		if (exchange.isInOptionalOutPattern() || exchange.isInOutPattern()) {
System.out.println("message inout");
			String fileName = exchange.getInMessageContentAsString();

			String getFormat = getExtensions().getPetalsExtensions()
					.getProperty("ftp.get.exchange-data", "source");

			if (fileName.startsWith("<file-name>")
					&& fileName.endsWith("</file-name>")) {

				Properties properties = getExtensions()
						.getPetalsExtensions();
				String server = properties
						.getProperty(FTPSUProperties.FTP_SERVER);
				int port = Integer.parseInt(properties.getProperty(
						FTPSUProperties.FTP_PORT,
						FTPSUProperties.DEFAULT_FTP_PORT.toString()));
				String login = properties
						.getProperty(FTPSUProperties.FTP_LOGIN);
				String pwd = properties
						.getProperty(FTPSUProperties.FTP_PASSWORD);

				String sName = fileName.substring("<file-name>".length(),
						fileName.length() - "</file-name>".length());

				if ("default".equals(sName)) {
					sName = getExtensions().getPetalsExtensions().getProperty(FTPSUProperties.FTP_FILENAME);
					if ("".equals(sName)){
						exchange.setFault(new Exception("the file-name in the service unit is empty"));
					}
				}



				if ("source".equalsIgnoreCase(getFormat)) {
					DataHandler dh = FTPUtils.get(server, port, login, pwd,
							sName);

					Source stream = new StreamSource(dh.getInputStream());

					exchange.setOutMessageContent(stream);

				} else if ("attachments".equalsIgnoreCase(getFormat)) {
					getLogger().info("get " + sName);
					DataHandler dh = FTPUtils.get(server, port, login, pwd,
							sName);
					exchange.getOutMessage().addAttachment(sName, dh);
					getLogger().info(sName + " attachment added");
					exchange
							.setOutMessageContent("<attached-files><file-name>"+sName+"</file-name></attached-files>");

				} else {
					throw new Exception(
							"The property 'GET-FORMAT' of the ServiceUnit is nor 'source' neither 'attachment' :"+getFormat);
				}

			} else {
				exchange.setFault(new Exception("Request IN parameter is invalid."));
			}
		} else {
			throw new Exception(
			"Exchange pattern for GET operation MUST be IN-OUt or OPTIONAL-IN-OUT");

		}

	}
	/**
	 * FTP DIR operation. <br>
	 * @param exchange
	 * 				the message exchange, MUST be non null
	 * @throws Exception
	 * 				impossible to realize the GET operation, or pattern is not InOut or InOptionalOut
	 */

	private void processDir(Exchange exchange) throws Exception {

		if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {

			Properties properties = getExtensions().getPetalsExtensions();
			String server = properties.getProperty(FTPSUProperties.FTP_SERVER);
			int port = Integer.parseInt(properties.getProperty(
					FTPSUProperties.FTP_PORT, FTPSUProperties.DEFAULT_FTP_PORT
							.toString()));
			String login = properties.getProperty(FTPSUProperties.FTP_LOGIN);
			String pwd = properties.getProperty(FTPSUProperties.FTP_PASSWORD);
			String destination = properties
					.getProperty(FTPSUProperties.FTP_FOLDER);
			List<String> fileNames = FTPUtils.list(server, port, login, pwd,
					destination);

			String result = "<list folder=\"" + destination + "\" >";
			for (String string : fileNames) {
				result += "<file-name>" + string + "</file-name>";
			}
			result += "</list>";
			exchange.setOutMessageContent(result);
		} else {
			throw new Exception(
					"Exchange pattern for PUT operation MUST be IN-OUT or IN-OPTIONAL-OUT");
		}
	}
/**
  * FTP MGET operation <br>
  * All files are getted as attachments
  *
  * @param exchange
  * @throws Exception
  */
	private void processMGet(Exchange exchange) throws Exception {

		if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
			getLogger().info("process MGET operation");

			Properties properties = getExtensions().getPetalsExtensions();
			String server = properties.getProperty(FTPSUProperties.FTP_SERVER);
			int port = Integer.parseInt(properties.getProperty(
					FTPSUProperties.FTP_PORT, FTPSUProperties.DEFAULT_FTP_PORT
							.toString()));
			String login = properties.getProperty(FTPSUProperties.FTP_LOGIN);
			String pwd = properties.getProperty(FTPSUProperties.FTP_PASSWORD);
			String destination = properties
					.getProperty(FTPSUProperties.FTP_FOLDER);

			Map<String, DataHandler> files = FTPUtils.mGet(server, port, login,
					pwd, destination);


			String response = "<attached-files>";

			for (String filename : files.keySet()) {

				exchange.getOutMessage().addAttachment(filename,
						files.get(filename));
				getLogger().info(filename+" added as attachment.");
				response +="<file-name>"+filename+"</file-name>";
			}

			response +="</attached-files>";
			exchange.setOutMessageContent(response);
		} else {
			throw new Exception(
					"Exchange pattern for MGET operation MUST be IN-OUT or IN-OPTIONAL-OUT");
		}
		getLogger().info("process MGET operation ended");
	}

	/**
	 *
	 *
	 * @param source
	 *            MUST be non null
	 * @throws NullPointerException
	 *             source is null
	 * @throws Exception
	 *             can not process the FTP PUT operation
	 */
	private void sendSource(Source source) throws Exception {
		if (source == null)
			throw new NullPointerException("source parameter is null");

		getLogger().info("Try to put XML Source.");



		Properties prop = getExtensions().getPetalsExtensions();
		String server = prop.getProperty(FTPSUProperties.FTP_SERVER);
		int port = Integer.parseInt(prop.getProperty(FTPSUProperties.FTP_PORT,
				FTPSUProperties.DEFAULT_FTP_PORT.toString()));
		String login = prop.getProperty(FTPSUProperties.FTP_LOGIN);
		String pwd = prop.getProperty(FTPSUProperties.FTP_PASSWORD);
		String folder = prop.getProperty(FTPSUProperties.FTP_FOLDER, "");
		String fileName = getExtensions().getPetalsExtensions().getProperty(
				"file-name", "source.xml");

		FTPUtils.putSource(fileName, source , server, port, login, pwd, folder);
		getLogger().info("XML Source put.");

	}

	/**
	  * Send an attachement.
	  * @param fileName
	  * @param content
	  */
	private void putAttachment(String fileName, DataHandler handler)
			throws Exception {
		if (fileName == null || handler == null)
			throw new NullPointerException(
					"filename or handler parameter is null");

		getLogger().info("Try to put " + fileName + " attachment.");
		putDatahandler(fileName, handler);
		getLogger().info(fileName + " put.");
	}

	/**
	 * Send the DataHandler to the FTP server, using FTPProperties with all the
	 * SU properties.
	 *
	 * @param fileName
	 * @param dh
	 *
	 */
	private void putDatahandler(String fileName, DataHandler dh)
			throws Exception {
		if (fileName == null || dh == null)
			throw new NullPointerException(
					"filename or inputStream parameter is null");

		Properties prop = getExtensions().getPetalsExtensions();
		String server = prop.getProperty(FTPSUProperties.FTP_SERVER);
		int port = Integer.parseInt(prop.getProperty(FTPSUProperties.FTP_PORT,
				FTPSUProperties.DEFAULT_FTP_PORT.toString()));
		String login = prop.getProperty(FTPSUProperties.FTP_LOGIN);
		String pwd = prop.getProperty(FTPSUProperties.FTP_PASSWORD);
		String folder = prop.getProperty(FTPSUProperties.FTP_FOLDER, "");

		FTPUtils.put(fileName, dh, server, port, login, pwd, folder);
	}

	}
